{% extends 'allianceauth/base.html' %}
{% load static %}
{% load i18n %}

{% block page_title %}{{ page_title }} - {% translate 'Moon Mining' %}{% endblock %}

{% block content %}
    <br>
    {% include 'moonmining/partials/menu.html' %}
    <h2>{{ page_title }}</h2>
    {% block details %}{% endblock %}
{% endblock %}

{% block extra_javascript %}
    {% include 'bundles/datatables-js.html' %}
    <script type="text/javascript" src="{% static 'moonmining/vendor/datatables/plugins/filterDropDown.min.js' %}"></script>
    {% include 'moonmining/partials/global_js.html' %}

{% endblock %}

{% block extra_css %}
    {% include 'bundles/datatables-css.html' %}
    <link href="{% static 'moonmining/css/global.css' %}" type="text/css" rel="stylesheet">
    <link href="{% static 'moonmining/css/moonmining.css' %}" type="text/css" rel="stylesheet">
{% endblock %}
