{% load humanize %}

<div role="tabpanel" class="tab-pane" id="tab_user_mining">
    <p class="report-title">Total mining of corporation moons by known mains per month</p>
    <div class="table-responsive">
        <table class="table table-striped table-width-fix" id="table_user_mining">
            <thead>
                <tr>
                    <th>Name</th>
                    <th>Corporation</th>
                    <th>State</th>
                    <th>{{ month_minus_3 }}<br>Volume</th>
                    <th>{{ month_minus_3 }}<br>Est.&nbsp;Value</th>
                    <th>{{ month_minus_2 }}<br>Volume</th>
                    <th>{{ month_minus_2 }}<br>Est.&nbsp;Value</th>
                    <th>{{ month_minus_1 }}<br>Volume</th>
                    <th>{{ month_minus_1 }}<br>Est.&nbsp;Value</th>
                    <th>{{ month_current }}<br>Volume</th>
                    <th>{{ month_current }}<br>Est.&nbsp;Value</th>
                </tr>
            </thead>
            <tfoot>
                <th class="info">GRAND&nbsp;TOTAL</th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
            </tfoot>
        </table>
    </div>
    <p class="text-muted">Last update of mining ledgers: {{ ledger_last_updated|naturaltime|default:"?" }}</p>
</div>
