<!-- Base template for creating the HTML for a modal. -->

{% load i18n %}

<div class="modal-dialog {{ modal_size|default:'modal-lg' }}" role="document" id="{{ modal_content_id }}">
    <div class="modal-content">
        <div class="modal-header">
            <button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">
                &times;
            </span></button>
            <h4 class="modal-title" id="myModalLabel">{{ modal_title }}</h4>
        </div>

        <div class="modal-body">
            {% include modal_body_url %}
        </div>

        <div class="modal-footer">
            {% if new_page_url %}
                <a class="btn btn-default pull-left" href="{{ new_page_url }}?new_page=yes" target="_blank" title="Open in new tab">
                    <i class="far fa-window-restore"></i>
                </a>
            {% endif %}
            <button type="button" class="btn btn-default" data-dismiss="modal">
                {% translate 'Close' %}
            </button>
        </div>
    </div>
</div>
