{% load i18n %}
{% load humanize %}
{% load moonmining %}

<dl class="dl-horizontal">
    <dt>Refinery:</dt>
    <dd>{{ extraction.refinery.name }}</dd>

    <dt>Company:</dt>
    <dd>{{ extraction.refinery.owner.name }}</dd>

    <dt>Moon:</dt>
    <dd>{{ extraction.refinery.moon }}</dd>

    <dt>Location:</dt>
    <dd>
        {% include "moonmining/partials/location.html" with eve_solar_system=extraction.refinery.moon.eve_moon.eve_planet.eve_solar_system %}
    </dd>

    <dt>Labels:</dt>
    <dd>{{ extraction.refinery.moon.labels_html }}</dd>
</dl>
