# Generated by Django 4.0.3 on 2022-04-22 18:19

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("moonmining", "0005_make_pricing_more_flexible"),
    ]

    operations = [
        migrations.CreateModel(
            name="Label",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("description", models.TextField(blank=True, default="")),
                ("name", models.CharField(max_length=100, unique=True)),
                (
                    "style",
                    models.CharField(
                        choices=[
                            ("primary", "dark blue"),
                            ("success", "green"),
                            ("default", "grey"),
                            ("info", "light blue"),
                            ("warning", "orange"),
                            ("danger", "red"),
                        ],
                        default="default",
                        max_length=16,
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="moon",
            name="label",
            field=models.ForeignKey(
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                to="moonmining.label",
            ),
        ),
    ]
