{% load humanize %}

<div role="tabpanel" class="tab-pane" id="tab_user_uploaded">
    <p class="report-title">Uploaded moons per main</p>
    <div class="table-responsive">
        <table class="table table-striped table-width-fix" id="table_user_uploaded">
            <thead>
                <tr>
                    <th>Name</th>
                    <th>Corporation</th>
                    <th>State</th>
                    <th>Count</th>
                </tr>
            </thead>
            <tfoot>
                <th class="info">GRAND&nbsp;TOTAL</th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
            </tfoot>
        </table>
    </div>
</div>
