<!-- {% url 'moonmining:extraction_ledger' extraction.pk as new_page_url %} -->
{% include "moonmining/modals/content_base.html" with modal_content_id="modalExtractionDetailsContent" modal_body_url="moonmining/partials/extraction_ledger.html" modal_title="Mining Ledger" new_page_url=False modal_size="modal-xl" %}
