{% load humanize %}
{% load moonmining %}

<div role="tabpanel" class="tab-pane" id="tab_{{ category }}">
    <div class="table-responsive">
        <table class="table table-striped table-extractions table-width-fix" id="table_{{ category }}">
            <thead>
                <th>Chunk arrival</th>
                <th>Refinery</th>
                <th>Location</th>
                <th>Labels</th>
                <th>Volume (m3)</th>
                <th>Est.&nbsp;Value</th>
                <th>Est.&nbsp;Mined</th>
                <th></th>
            </thead>
            <tfoot>
                <th>TOTAL</th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
                <th></th>
            </tfoot>
        </table>
    </div>
</div>
