/* Moon list */
.table-moons thead th {
	text-align: left;
	vertical-align: middle;
}

.table-moons tbody td {
	text-align: left;
	vertical-align: middle !important;
}

.table-moons tbody td:nth-child(4) {
	white-space: nowrap;
}

.table-moons tbody td:nth-child(5) {
	white-space: nowrap;
}

.table-moons thead th:nth-child(6) {
	text-align: right;
}

.table-moons tbody td:nth-child(6) {
	text-align: right;
}


.table-moons tbody td:nth-child(7) {
	text-align: right;
	white-space: nowrap;
}
