{% load i18n %}
{% load bootstrap %}
{% load static %}

<div role="tabpanel" class="tab-pane" id="tab_{{ category }}">
    <div id="loader_{{ category }}">
        {% if NIGHT_MODE %}
            <img src="{% static 'moonmining/img/Spinner-1s-64px-dark.gif' %}" alt="Spinner"/>
        {% else %}
            <img src="{% static 'moonmining/img/Spinner-1s-64px-light.gif' %}" alt="Spinner"/>
        {% endif %}
    </div>

    <div id="wrapper_{{ category }}" style="display: none !important;">
        <div class="table-responsive">
            <table class="table table-striped table-moons table-width-fix" id="table_{{ category }}">
                <thead>
                    <th>Moon</th>
                    <th>System</th>
                    <th>Const./Region</th>
                    <th>Refinery</th>
                    <th>Labels</th>
                    <th>Est.&nbsp;Value</th>
                    <th></th>
                </thead>
            </table>
        </div>
    </div>
</div>
