<div role="tabpanel" class="tab-pane active" id="tab_owned_value">
    <p class="report-title">Potential monthly income from owned moons by corporation</p>
    <div class="table-responsive">
        <table class="table table-striped table-width-fix" id="table_owned_value">
            <thead>
                <th></th>
                <th>Corporation / Moon</th>
                <th>Region</th>
                <th>Rarity</th>
                <th>Est.&nbsp;Value</th>
                <th>% of Grand Total *</th>
                <th>Rank of Grand Total *</th>
                <th>Totals</th>
            </thead>
            <tfoot>
                <th></th>
                <th class="info">GRAND TOTAL</th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
                <th class="info"></th>
            </tfoot>
        </table>
    </div>
    {% include 'moonmining/partials/value_estimate_legend.html' %}
</div>
